/********************************** (C) COPYRIGHT *******************************
* File Name          : IIC.h
* Author             : WCH
* Version            : V1.0
* Date               : 2018/12/12
* Description        :
*******************************************************************************/

#ifndef __IIC_H__
#define __IIC_H__

#ifdef __cplusplus
extern "C" {
#endif
#include "typedef.h"
#include "driver_gpio.h"

#define   IIC_SDA     (GPIO_PIN_3)  //PA3
#define   IIC_SCL     (GPIO_PIN_2)  //PA2
#define   RST_PIN     (GPIO_PIN_1)  //PA1
#define   INT_PIN     (GPIO_PIN_0)   //PA0

void Touch_IIC_Init(void);
BOOL GT_Check_I2C_Address(UINT8 Address);
UINT8 IICReadOneByte(UINT32 ReadAddr);
BOOL IICReadBytes(UINT32 ReadAddr,UINT8 *pBuffer,UINT16 Length);
BOOL IICWriteOneByte(UINT32 WriteAddr,UINT8 DataToWrite);
BOOL IICWriteBytes(UINT32 WriteAddr,UINT8 *pBuffer,UINT16 Length);

#ifdef __cplusplus
}
#endif

#endif
